CHROMIUM_COMPONENT_IDS = {
  "autofill_states_data":                       "eeigpngbgcognadeebkilcpcaedhellh",
  "pki_metadata":                               "efniojlnjndmcbiieegkicadnoecjjef",
  "subresource_filter_rules":                   "gcmjkmgdlgnkkcocmoeiminaijmmjnii",
  "crowd_deny":                                 "ggkkehgbnfjpeggfpleeakpidbkibbmn",
  "certificate_error_assistant":                "giekcmmlnklenlaomppkphknjmnnpneh",
  "related_website_sets":                       "gonpemdgkjcecdgbnaabipppbmgfggbe",
  "amount_extraction_heuristic_regexes":        "hajigopbbjhghbfimgkfmpenfkclmohk",
  "crlset":                                     "hfnkpimlhhgieaddgfemjhofmfblmnib",
  "hyphenation":                                "jamhcnnkihinmdlkakkaopbjbbcngflc",
  "third_party_cookie_deprecation_metadata":    "jflhchccmppkfebkiaminageehmchikm",
  "safety_tips":                                "jflookgnkcckhobaglndicnbbgbonegd",
  "file_type_policies":                         "khaoiebndkojlmppeemjhbpbandiljpe",
  "trust_token_key_commitments":                "kiabhabjdbkjdpjbpigfodbdjmbglcoo",
  "mei_preload":                                "laoigpblnllgcgjnjnllmfolckpjlhki",
  "origin_trials":                              "llkgjffcdpffmhiakmfcdcblohccpfmo",
  "optimization_hints":                         "lmelglejhemejginpboagddgdfbepgmp",
  "cookie_readiness_list":                      "mcfjlbnicoclaecapilmleaelokfnijm",
  "screenai_library":                           "mfhmdacoffpmifoibamicehhklffanao",
  "privacy_sandbox_attestations":               "niikhdgajlphfehepabhhblakbdgeefj",
  "ondeviceheadsuggest":                        "obedbbhbpmojnkanicioggnmelmoomoc",
  "widevine_content_decryption_module":         "oimompecagnajdejgnnjijobebaeigek",
  "zxcvbn_data_dictionaries":                   "ojhpjlocmbogdgmfpkhlaaeamibhnphh",
  "open_cookie_database":                       "pmagihnlncbcefglppponlgakiphldeh"
}